﻿# Parameters
    param (
        $FileName
    )

# Open file dialog if no filename was given
    if (!($FileName)) {
        [System.Reflection.Assembly]::LoadWithPartialName("System.windows.forms") | Out-Null
    
        $OpenFileDialog = New-Object System.Windows.Forms.OpenFileDialog
        $OpenFileDialog.initialDirectory = $pwd
        $OpenFileDialog.filter = "TMX (*.tmx)| *.tmx|All files (*.*)| *.*"
        $OpenFileDialog.Title = "Select the TMX file you want to fix"
        $OpenFileDialog.ShowDialog() | Out-Null
    }

# Check path
    $filename = $OpenFileDialog.filename | Resolve-Path

# Define functions
    # Function for reporting errors
        function ErrorBreak {
            param (
                $ErrorMessage
            )

            Write-host "Error!"
            Write-host $ErrorMessage

            # Wait for user to press any key before ending script
            Write-Host "Press any key to close this window..."

            $x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")

            break
        }

# Disable notifications of errors to the user, since this will be done by this script
    $ErrorActionPreference = 'silentlycontinue'

#------------- Begin work! -------------
    # Read file contents
    $FileContent = Get-Content -Path $FileName -raw
    if (!($?)) {ErrorBreak "The file was not found, or could not be read."} # Check if reading the file completed correctly, and report the error if it didn't

    # Check if the file contains the tag that specifies it is a TMX file
    if (!($FileContent -match '<tmx version=".+?">')) {ErrorBreak "This does not appear to be a TMX translation memory."}

    # Replace the undesired entities
    $FileContent = $FileContent -replace '&#x1?[01-9A-F];',' ' # This will replace unicode characters nº 1 to 1F, which are special characters not allowed in TMX

    # Check if the file has malformed XML
    [xml]$FileContent | out-null
    if (!($?)) {ErrorBreak "The underlying XML code of the file is malformed or otherwise incorrect. Attempt to obtain a well-formed copy."}

    #Save the text to the original file
    #$FileContent | Set-Content -Path ($FileName.replace('.tmx','.repaired.tmx')) -Encoding UTF8
    write-host ("Writing file " + $FileName.path.replace('.tmx','.repaired.tmx'))
    ([xml]$FileContent).save($FileName.path.replace('.tmx','.repaired.tmx'))
    
    # Check if the save was done correctly
    if ($?) {Write-host "Offending characters were successfully removed."}
    else {ErrorBreak "The file could not be saved."}

    # Wait for user to press any key before ending script
    Write-Host "Press any key to close this window..."

    $x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")